.compact-header[data-v-481c0e50]{padding:4px 0}.schema-card[data-v-481c0e50]{border:1px solid rgba(13,148,136,.1);border-radius:16px}.rerun-alert[data-v-481c0e50]{border-radius:16px}.schema-table th[data-v-481c0e50]{font-weight:600!important;text-transform:uppercase;font-size:.7rem!important;letter-spacing:.5px;background:#0d948808}.field-name[data-v-481c0e50]{font-weight:600;font-size:.95rem}[data-v-481c0e50] .v-theme--dark .schema-card{border-color:#0d948833}[data-v-481c0e50] .v-theme--dark .schema-table th{color:#ffffff80!important;background:#0d94880f}
