.filter-card[data-v-800bb8d0],.notifications-card[data-v-800bb8d0]{border:1px solid rgba(13,148,136,.1);border-radius:16px}.unread-notification[data-v-800bb8d0]{border-left:3px solid rgb(var(--v-theme-primary));background:rgba(var(--v-theme-primary),.06)}.grouped-notification[data-v-800bb8d0]{background:rgba(var(--v-theme-surface-variant),.3)}
