.settings-card[data-v-f7db47d0]{border:1px solid rgba(13,148,136,.1)}.dialog-header[data-v-f7db47d0]{background:#0d9488;color:#fff}.key-display[data-v-f7db47d0] input{font-family:monospace;font-size:.85rem}
