.compact-header[data-v-a10ba821]{padding:4px 0}.jobs-card[data-v-a10ba821]{border:1px solid rgba(13,148,136,.1);border-radius:16px;overflow:hidden}.jobs-table[data-v-a10ba821]{border-radius:0}.jobs-table[data-v-a10ba821] thead th{font-weight:600!important;text-transform:uppercase;font-size:.7rem!important;letter-spacing:.5px;background:#0d948808}.schedule-toggle[data-v-a10ba821]{width:100%}.schedule-toggle .v-btn[data-v-a10ba821]{flex:1}.edit-dialog[data-v-a10ba821],.history-dialog[data-v-a10ba821]{border-radius:16px!important;overflow:hidden}.history-table[data-v-a10ba821] thead th{font-weight:600!important;text-transform:uppercase;font-size:.7rem!important;letter-spacing:.5px}[data-v-a10ba821] .v-theme--dark .jobs-card{border-color:#0d948833}[data-v-a10ba821] .v-theme--dark .jobs-table thead th{background:#0d94881a}[data-v-a10ba821] .v-theme--dark .compact-header{border-bottom-color:#0d948833}
