import{A as a}from"./index-jHWjNw8g.js";async function c(){return(await a.get("/jobs")).data.data}async function r(t,s){return(await a.get(`/models/${t}/versions/${s}/jobs`)).data.data}async function p(t){return(await a.get(`/jobs/${t}`)).data.data}async function u(t,s){return(await a.patch(`/jobs/${t}`,s)).data.data}async function i(t){await a.post(`/jobs/${t}/trigger`)}async function d(t){return(await a.get(`/jobs/${t}/runs`)).data.data}async function g(t,s){const o=(await r(t,s)).filter(n=>n.is_active);return await Promise.all(o.map(n=>i(n.id))),o.length}async function f(t){return(await a.post(`/jobs/${t}/backfill`)).data.data}async function l(t=1,s=10){return(await a.get("/drift-overview",{params:{page:t,page_size:s}})).data}const w={list:c,listForVersion:r,get:p,update:u,trigger:i,triggerAllForVersion:g,getHistory:d,backfill:f,getDriftOverview:l};export{w as j};
