import{d as _,j as n,b,w as i,m as d,r as m,o as s,F as I,p as f,v as h,c as N,f as $,t as x,_ as y}from"./index-jHWjNw8g.js";const V={class:"version-nav mb-4"},k=_({__name:"VersionNavTabs",props:{modelId:{},versionId:{}},setup(c){const e=c,l=h(),a=d(()=>[{title:"Dashboard",icon:"mdi-view-dashboard-outline",routeName:"dashboard",to:`/models/${e.modelId}/versions/${e.versionId}/dashboard`},{title:"Compare",icon:"mdi-compare",routeName:"comparison",to:`/models/${e.modelId}/versions/${e.versionId}/compare`},{title:"Drift",icon:"mdi-chart-timeline-variant-shimmer",routeName:"version-drift-results",to:`/models/${e.modelId}/versions/${e.versionId}/drift`},{title:"Jobs",icon:"mdi-timer-sync-outline",routeName:"version-jobs",to:`/models/${e.modelId}/versions/${e.versionId}/jobs`},{title:"Schema",icon:"mdi-file-tree",routeName:"version-schema",to:`/models/${e.modelId}/versions/${e.versionId}/schema`}]),v=d(()=>{const t=a.value.findIndex(r=>r.routeName===l.name);return t>=0?t:0});return(t,r)=>{const u=m("v-tab"),p=m("v-tabs");return s(),n("div",V,[b(p,{"model-value":v.value,color:"primary",density:"comfortable"},{default:i(()=>[(s(!0),n(I,null,f(a.value,o=>(s(),N(u,{key:o.routeName,to:o.to,"prepend-icon":o.icon,class:"nav-tab"},{default:i(()=>[$(x(o.title),1)]),_:2},1032,["to","prepend-icon"]))),128))]),_:1},8,["model-value"])])}}}),T=y(k,[["__scopeId","data-v-5dc28439"]]);export{T as V};
